from .ActuatorCommand import ActuatorCommand
from .ActuatorCommandType import ActuatorCommandType
from .ActuatorState import ActuatorState
from .ActuatorStatus import ActuatorStatus
from .Alarm import Alarm
from .ConfigurationCommand import ConfigurationCommand
from .ConfigurationCommandType import ConfigurationCommandType
from .Device import Device
from .FileTransferPacket import FileTransferPacket
from .FirmwareCommand import FirmwareCommand
from .FirmwareCommandType import FirmwareCommandType
from .FirmwareErrorType import FirmwareErrorType
from .FirmwareStatus import FirmwareStatus
from .FirmwareStatusType import FirmwareStatusType
from .FirmwareUpdateStateType import FirmwareUpdateStateType
from .InboundMessage import InboundMessage
from .OutboundMessage import OutboundMessage
from .SensorReading import SensorReading

__all__ = [
    "ActuatorCommand",
    "ActuatorCommandType",
    "ActuatorState",
    "ActuatorStatus",
    "Alarm",
    "ConfigurationCommand",
    "ConfigurationCommandType",
    "Device",
    "FileTransferPacket",
    "FirmwareCommand",
    "FirmwareCommandType",
    "FirmwareErrorType",
    "FirmwareStatus",
    "FirmwareStatusType",
    "FirmwareUpdateStateType",
    "InboundMessage",
    "OutboundMessage",
    "SensorReading",
]



#   Copyright 2018 WolkAbout Technology s.r.o.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""Device Module."""

from wolk import LoggerFactory


class Device:
    """
    Hold all necessary information to communicate with WolkAbout IoT Platform.

    :ivar actuator_references: Actuator references enabled on device
    :vartype actuator_references: list
    :ivar key: Device key obtained when creating device on WolkAbout IoT Platform
    :vartype key: str
    :ivar logger: Logger instance issued by wolk.LoggerFactory
    :vartype logger: logging.Logger
    :ivar password: Password obtained when creating device on WolkAbout IoT Platform
    :vartype password: str

    """

    def __init__(self, key, password, actuator_references=None):
        """
        Contain device credentials necessary for connection to WolkAbout IoT Platform.

        :param key: Device key obtained when creating device on WolkAbout IoT Platform
        :type key: str
        :param password: Password obtained when creating device on WolkAbout IoT Platform
        :type password: str
        :param actuator_references: Actuator references enabled on device
        :type actuator_references: list or None
        """
        self.key = key
        self.password = password
        if actuator_references:
            self.actuator_references = actuator_references
        else:
            self.actuator_references = list()
        self.logger = LoggerFactory.logger_factory.get_logger(
            str(self.__class__.__name__)
        )
        self.logger.debug(
            "Init - Key: %s ; Password: %s ; Actuator references: %s",
            key,
            password,
            actuator_references,
        )



#   Copyright 2018 WolkAbout Technology s.r.o.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

from abc import ABC, abstractmethod

"""
ConnectivityService Module.
"""


class ConnectivityService(ABC):
    """Enable communication with WolkAbout IoT Platform."""

    @abstractmethod
    def connect(self):
        """Connect to WolkAbout IoT Platform."""
        pass

    @abstractmethod
    def disconnect(self):
        """Disconnect from WolkAbout IoT Platform."""
        pass

    @abstractmethod
    def publish(self, outbound_message):
        """
        Publish a message to WolkAbout IoT Platform.

        Returns true on success, false otherwise.

        :param outbound_message: Message to send
        :type outbound_message: wolk.models.OutboundMessage.OutboundMessage
        :returns: success
        :rtype: bool
        """
        pass

    @abstractmethod
    def set_inbound_message_listener(self, listener):
        """
        Set a callback to ``WolkConnect._on_inbound_message`` method.

        callback function passes back an ``wolk.models.InboundMessage.InboundMessage`` object.

        :param listener: WolkConnect instance
        :type listener: wolk.WolkConnect.WolkConnect
        """
        pass



#   Copyright 2018 WolkAbout Technology s.r.o.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

from abc import ABC, abstractmethod

"""FirmwareURLDownloadHandler Module."""


class FirmwareURLDownloadHandler(ABC):
    """Responsible for providing means of downloading firmware file from a URL."""

    @abstractmethod
    def download(self, url, file, result_callback):
        """
        Download the resource located at the url to the file.

        Return the result of the download process as a bool to result_callback

        :param url: URL from where to download the file
        :type url: str
        :param file: Name of file to which to download
        :type file: str
        :param result_callback: Function to which to report the outcome
        :type result_callback: function
        """
        pass



#   Copyright 2018 WolkAbout Technology s.r.o.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

from abc import ABC, abstractmethod

"""
FirmwareUpdate Module.
"""


class FirmwareUpdate(ABC):
    """
    Firmware Update enabler.

    Responsible for handling file and URL download, as well as reporting firmware status.
    Should be implemented to depend upon an object of FirmwareHandler class
    """

    @abstractmethod
    def handle_file_upload(self, firmware_command):
        """Pass the information necessary to start a file download to an instance of FirmwareHandler."""
        pass

    @abstractmethod
    def handle_url_download(self, firmware_command):
        """Pass the information necessary to start a URL download to an instance of FirmwareHandler."""
        pass

    @abstractmethod
    def handle_url_download_result(self, result):
        """Handle the result of the url file download reported from an instance of FirmwareHandler."""
        pass

    @abstractmethod
    def handle_install(self):
        """Pass the install command to an instance of FirmwareHandler."""
        pass

    @abstractmethod
    def handle_abort(self):
        """Pass the abort command to an instance of FirmwareHandler."""
        pass

    @abstractmethod
    def handle_packet(self, packet):
        """Pass the packet to an instance of FirmwareHandler."""
        pass

    @abstractmethod
    def report_result(self):
        """Report the results of the firmware update process."""
        pass

    @abstractmethod
    def set_on_file_packet_request_callback(self, on_file_packet_request_callback):
        """Set a callback function to handle file packet requests."""
        pass

    @abstractmethod
    def set_on_status_callback(self, on_status_callback):
        """Set a callback fucntion to handle firmware status reporting."""
        pass



#   Copyright 2018 WolkAbout Technology s.r.o.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

from abc import ABC, abstractmethod

"""
ActuationHandler Module.
"""


class ActuationHandler(ABC):
    """Handle actuation commands received from WolkAbout IoT platform."""

    @abstractmethod
    def handle_actuation(self, reference, value):
        """
        Set actuator to value.

        When the actuation command is given from the platform, it will be delivered to this method.
        This method should pass the new value to the device's actuator.
        Must be implemented as non blocking.
        Must be implemented as thread safe.

        :param reference: Reference of the actuator
        :type reference: str
        :param value: Value to which to set the actuator
        :type value: int or float or str
        """
        pass



#   Copyright 2018 WolkAbout Technology s.r.o.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

from abc import ABC, abstractmethod


"""
ConfigurationHandler Module.
"""


class ConfigurationHandler(ABC):
    """Set device's configuration options."""

    @abstractmethod
    def handle_configuration(self, configuration):
        """
        Change device's configuration options.

        When the configuration command is given from the platform, it will be delivered to this method.
        This function should update device configuration with received configuration values.
        Must be implemented as non blocking.
        Must be implemented as thread safe.

        :param configuration: Configuration option reference:value pairs
        :type configuration: dict
        """
        pass



#   Copyright 2018 WolkAbout Technology s.r.o.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""
ActuatorStatus Module.
"""


class ActuatorStatus:
    """Current status of an actuator."""

    def __init__(self, reference, state, value):
        """
        Information about actuator status.

        :param reference: Reference of the actuator
        :type reference: str
        :param state: Actuators current state
        :type state: wolk.models.ActuatorState.ActuatorState
        :param value: Actuators current value
        :type value: int or float or str
        """
        self.reference = reference
        self.state = state
        self.value = value



# coding=utf-8
"""
.. module:: wolk

This module provides connection to WolkAbout IoT Platform.

To start publishing data to the platform
create an instance of Device class with credentials obtained from the platform
and pass it to an instance of WolkConnect class.

For more information about module features visit:
https://github.com/Wolkabout/WolkConnect-Python/tree/master/examples/full_feature_set
"""
from .models.ActuatorCommand import ActuatorCommand
from .models.ActuatorCommandType import ActuatorCommandType
from .models.ActuatorState import ActuatorState
from .models.ActuatorStatus import ActuatorStatus
from .models.Alarm import Alarm
from .models.ConfigurationCommand import ConfigurationCommand
from .models.ConfigurationCommandType import ConfigurationCommandType
from .models.Device import Device
from .models.FileTransferPacket import FileTransferPacket
from .models.FirmwareCommand import FirmwareCommand
from .models.FirmwareCommandType import FirmwareCommandType
from .models.FirmwareErrorType import FirmwareErrorType
from .models.FirmwareStatus import FirmwareStatus
from .models.FirmwareStatusType import FirmwareStatusType
from .models.FirmwareUpdateStateType import FirmwareUpdateStateType
from .models.InboundMessage import InboundMessage
from .models.OutboundMessage import OutboundMessage
from .models.Protocol import Protocol
from .models.SensorReading import SensorReading
from .interfaces.ActuationHandler import ActuationHandler
from .interfaces.ActuatorStatusProvider import ActuatorStatusProvider
from .interfaces.ConfigurationHandler import ConfigurationHandler
from .interfaces.ConfigurationProvider import ConfigurationProvider
from .interfaces.ConnectivityService import ConnectivityService
from .interfaces.FirmwareInstaller import FirmwareInstaller
from .interfaces.FirmwareURLDownloadHandler import FirmwareURLDownloadHandler
from .interfaces.InboundMessageDeserializer import InboundMessageDeserializer
from .interfaces.OutboundMessageFactory import OutboundMessageFactory
from .interfaces.OutboundMessageQueue import OutboundMessageQueue
from .FileSystemFirmwareHandler import FileSystemFirmwareHandler
from .LoggerFactory import logging_config
from .WolkConnect import WolkConnect


__all__ = [
    "ActuatorCommand",
    "ActuatorCommandType",
    "ActuatorState",
    "ActuatorStatus",
    "Alarm",
    "ConfigurationCommand",
    "ConfigurationCommandType",
    "Device",
    "FileTransferPacket",
    "FirmwareCommand",
    "FirmwareCommandType",
    "FirmwareErrorType",
    "FirmwareStatus",
    "FirmwareStatusType",
    "FirmwareUpdateStateType",
    "InboundMessage",
    "OutboundMessage",
    "Protocol",
    "SensorReading",
    "ActuationHandler",
    "ActuatorStatusProvider",
    "ConfigurationHandler",
    "ConfigurationProvider",
    "ConnectivityService",
    "FileSystemFirmwareHandler",
    "FirmwareInstaller",
    "FirmwareURLDownloadHandler",
    "logging_config",
    "InboundMessageDeserializer",
    "OutboundMessageFactory",
    "OutboundMessageQueue",
    "WolkConnect",
]



#   Copyright 2018 WolkAbout Technology s.r.o.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""OS Keep Alive Service module."""

from wolk import LoggerFactory
from wolk.interfaces.KeepAliveService import KeepAliveService
from threading import Timer


class OSKeepAliveService(KeepAliveService):
    """
    Send messages to platform in regular intervals to keep device connected.

    Used for cases where no data is being sent by device for over 30 minutes.

    :ivar connectivity_service: Connectivity service used to publish
    :vartype connectivity_service: wolk.interfaces.ConnectivityService.ConnectivityService
    :ivar interval: Number of seconds between each keep alive message
    :vartype interval: int
    :ivar logger: Logger instance issued by wolk.LoggerFactory
    :vartype logger: logging.Logger
    :ivar outbound_message_factory: message factory used to create pings
    :vartype outbound_message_factory: wolk.interfaces.OutboundMessageFactory.OutboundMessageFactory
    :ivar timer: timer for sending ping messages
    :vartype timer: wolk.OSKeepAliveService.RepeatingTimer
    """

    def __init__(self, connectivity_service, outbound_message_factory, interval=600):
        """
        Service for sending keep alive messages.

        :param connectivity_service: Connectivity service used to publish
        :type connectivity_service: wolk.interfaces.ConnectivityService.ConnectivityService
        :param outbound_message_factory: Message factory used to create pings
        :type outbound_message_factory: wolk.interfaces.OutboundMessageFactory.OutboundMessageFactory
        :param interval: Number of seconds between each keep alive message
        :type interval: int or None
        """
        self.logger = LoggerFactory.logger_factory.get_logger(
            str(self.__class__.__name__)
        )
        self.logger.debug(
            "Init:  Connectivity service: %s ; Outbound message factory: %s "
            "Interval: %s",
            connectivity_service,
            outbound_message_factory,
            interval,
        )
        self.connectivity_service = connectivity_service
        self.outbound_message_factory = outbound_message_factory
        self.interval = interval
        self.timer = None

    def handle_pong(self):
        """Handle keep alive response message received from the platform."""
        self.logger.debug("handle_pong called")
        pass

    def start(self):
        """
        Send a keep alive message as soon as the device is connected.

        Starts a repeating timer to send subsequent keep alive messages
        every self.interval seconds
        """
        self.logger.debug("start called")

        self.timer = RepeatingTimer(self.interval, self.send_keep_alive)
        t = Timer(5, self.timer.start)
        t.start()

    def stop(self):
        """Stop the repeating timer."""
        self.logger.debug("stop called")
        self.timer.cancel()

    def send_keep_alive(self):
        """
        Make ping and send.

        Create a keep alive message from the outbound message factory
        and publishes it using the connectivity service
        """
        self.logger.debug("send_keep_alive called")
        message = self.outbound_message_factory.make_from_keep_alive_message()
        self.connectivity_service.publish(message)


class RepeatingTimer:
    """
    Repeating timer calls function with arguments every interval.

    :ivar args: list of arguments
    :vartype args: list
    :ivar f: function to call
    :vartype f: function
    :ivar interval: number of seconds after which to call function
    :vartype interval: int
    :ivar kwargs: keyword arguments
    :vartype kwargs: kwargs
    :ivar timer: timer
    :vartype timer: threading.Timer
    """

    def __init__(self, interval, f, *args, **kwargs):
        """
        Create a repeating timer.

        :param interval: number of seconds after which to call function
        :type interval: int
        :param f: function to call
        :type f: function
        :param \*args: list of arguments
        :type \*args: list
        :param \*\*kwargs: keyword arguments
        :type \*\*kwargs: kwargs
        """
        self.interval = interval
        self.f = f
        self.args = args
        self.kwargs = kwargs

        self.timer = None

    def callback(self):
        """Call function and start timer."""
        self.f(*self.args, **self.kwargs)
        self.start()

    def cancel(self):
        """Cancel timer."""
        self.timer.cancel()

    def start(self):
        """Start timer."""
        self.timer = Timer(self.interval, self.callback)
        self.timer.start()



#   Copyright 2018 WolkAbout Technology s.r.o.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""JsonProtocolOutboundMessageFactory Module."""

from wolk.models.ActuatorState import ActuatorState
from wolk.models.FirmwareErrorType import FirmwareErrorType
from wolk.models.FirmwareStatusType import FirmwareStatusType
from wolk.models.OutboundMessage import OutboundMessage
from wolk.interfaces.OutboundMessageFactory import OutboundMessageFactory
from wolk import LoggerFactory


class JsonProtocolOutboundMessageFactory(OutboundMessageFactory):
    """
    Serialize messages to be sent to WolkAbout IoT Platform.

    :ivar device_key: Device key to use when serializing messages
    :vartype device_key: str
    :ivar logger: Logger instance issued by wolk.LoggerFactory
    :vartype logger: logging.Logger
    """

    def __init__(self, device_key):
        """
        Create a factory for serializing messages.

        :param device_key: Device key to use when serializing messages
        :type device_key: str
        """
        self.device_key = device_key
        self.logger = LoggerFactory.logger_factory.get_logger(
            str(self.__class__.__name__)
        )
        self.logger.debug("Init - Device key: %s", device_key)

    def make_from_sensor_reading(self, reading):
        """
        Serialize the sensor reading to be sent to WolkAbout IoT Platform.

        :param reading: Sensor reading to be serialized
        :type reading: wolk.models.SensorReading.SensorReading
        :returns: message
        :rtype: wolk.models.OutboundMessage.OutboundMessage
        """
        self.logger.debug("make_from_sensor_reading called")

        if isinstance(reading.value, tuple):

            delimiter = ","

            values_list = list()

            for value in reading.value:
                if value is True:
                    value = "true"
                elif value is False:
                    value = "false"
                if "\n" in str(value):
                    value = value.replace("\n", "\\n")
                    value = value.replace("\r", "")
                if '"' in str(value):
                    value = value.replace('"', '\\"')
                values_list.append(value)
                values_list.append(delimiter)

            values_list.pop()

            reading.value = "".join(map(str, values_list))

        if reading.value is True:
            reading.value = "true"
        elif reading.value is False:
            reading.value = "false"

        if "\n" in str(reading.value):
            reading.value = reading.value.replace("\n", "\\n")
            reading.value = reading.value.replace("\r", "")
        if '"' in str(reading.value):
            reading.value = reading.value.replace('"', '\\"')

        if reading.timestamp is None:
            message = OutboundMessage(
                "d2p/sensor_reading/d/" + self.device_key + "/r/" + reading.reference,
                '{"data":"' + str(reading.value) + '"}',
            )
        else:
            message = OutboundMessage(
                "d2p/sensor_reading/d/" + self.device_key + "/r/" + reading.reference,
                '{"utc":"'
                + str(reading.timestamp)
                + '","data":"'
                + str(reading.value)
                + '"}',
            )

        self.logger.debug(
            "make_from_sensor_reading - Channel: %s ; Payload: %s",
            message.channel,
            message.payload,
        )
        return message

    def make_from_alarm(self, alarm):
        """
        Serialize the alarm to be sent to WolkAbout IoT Platform.

        :param alarm: Alarm event to be serialized
        :type alarm: wolk.models.Alarm.Alarm
        :returns: message
        :rtype: wolk.models.OutboundMessage.OutboundMessage
        """
        self.logger.debug("make_from_alarm called")

        if alarm.active is True:
            alarm.active = "ON"
        elif alarm.active is False:
            alarm.active = "OFF"

        if alarm.timestamp is None:
            message = OutboundMessage(
                "d2p/events/d/" + self.device_key + "/r/" + alarm.reference,
                '{"data":"' + str(alarm.active) + '"}',
            )

        else:
            message = OutboundMessage(
                "d2p/events/d/" + self.device_key + "/r/" + alarm.reference,
                '{"utc":"'
                + str(alarm.timestamp)
                + '","data":"'
                + str(alarm.active)
                + '"}',
            )
        self.logger.debug(
            "make_from_alarm - Channel: %s ; Payload: %s",
            message.channel,
            message.payload,
        )
        return message

    def make_from_actuator_status(self, actuator):
        """
        Serialize the actuator status to be sent to WolkAbout IoT Platform.

        :param actuator: Actuator status to be serialized
        :type actuator: wolk.models.ActuatorStatus.ActuatorStatus
        :returns: message
        :rtype: wolk.models.OutboundMessage.OutboundMessage
        """
        self.logger.debug("make_from_actuator_status called")
        if actuator.state == ActuatorState.READY:
            actuator.state = "READY"

        elif actuator.state == ActuatorState.BUSY:
            actuator.state = "BUSY"

        elif actuator.state == ActuatorState.ERROR:
            actuator.state = "ERROR"

        if actuator.value is True:
            actuator.value = "true"
        elif actuator.value is False:
            actuator.value = "false"

        if "\n" in str(actuator.value):
            actuator.value = actuator.value.replace("\n", "\\n")
            actuator.value = actuator.value.replace("\r", "")
        if '"' in str(actuator.value):
            actuator.value = actuator.value.replace('"', '\\"')

        message = OutboundMessage(
            "d2p/actuator_status/d/" + self.device_key + "/r/" + actuator.reference,
            '{"status":"' + actuator.state + '","value":"' + str(actuator.value) + '"}',
        )
        self.logger.debug(
            "make_from_actuator_status - Channel: %s ; Payload: %s",
            message.channel,
            message.payload,
        )
        return message

    def make_from_firmware_status(self, firmware_status):
        """
        Serialize the firmware status to be sent to WolkAbout IoT Platform.

        :param firmware_status: Firmware status to be serialized
        :type firmware_status: wolk.models.FirmwareStatus.FirmwareStatus
        :returns: message
        :rtype: wolk.models.OutboundMessage.OutboundMessage
        """
        pass

    def make_from_chunk_request(self, file_name, chunk_index, chunk_size):
        """
        Request a firmware file chunk from WolkAbout IoT Platform.

        :param file_name: Name of the file to which the chunk belongs
        :type file_name: str
        :param chunk_index: Index of the requested chunk
        :type chunk_index: int
        :param chunk_size: Size of the requested chunk
        :type chunk_size: int
        :returns: message
        :rtype: wolk.models.OutboundMessage.OutboundMessage
        """
        pass

    def make_from_firmware_version(self, version):
        """
        Report the current version of firmware to WolkAbout IoT Platform.

        :param version: Firmware version to report
        :type version: str
        :returns: message
        :rtype: wolk.models.OutboundMessage.OutboundMessage
        """
        pass

    def make_from_keep_alive_message(self):
        """
        Create a ping message.

        :returns: message
        :rtype: wolk.models.OutboundMessage.OutboundMessage
        """
        pass

    def make_from_configuration(self, configuration):
        """
        Report device's configuration to WolkAbout IoT Platform.

        :param configuration: Device's current configuration
        :type configuration: dict

        :returns: message
        :rtype: wolk.models.OutboundMessage.OutboundMessage
        """
        self.logger.debug("make_from_configuration called")
        values = str()

        for reference, value in configuration.items():

            if isinstance(value, tuple):

                delimiter = ","

                values_list = list()

                for single_value in value:
                    if single_value is True:
                        single_value = "true"
                    elif single_value is False:
                        single_value = "false"
                    if "\n" in str(single_value):
                        single_value = single_value.replace("\n", "\\n")
                        single_value = single_value.replace("\r", "")
                    values_list.append(single_value)
                    values_list.append(delimiter)

                values_list.pop()
                value = "".join(map(str, values_list))

            if value is True:
                value = "true"
            elif value is False:
                value = "false"

            if "\n" in str(value):
                value = value.replace("\n", "\\n")
                value = value.replace("\r", "")

            values += '"' + reference + '":"' + str(value) + '",'

        values = values[:-1]

        message = OutboundMessage(
            "d2p/configuration_get/d/" + self.device_key, '{"values":{' + values + "}}"
        )
        self.logger.debug(
            "make_from_configuration - Channel: %s ; Payload: %s",
            message.channel,
            message.payload,
        )
        return message



#   Copyright 2018 WolkAbout Technology s.r.o.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

from abc import ABCMeta, abstractmethod

"""
InboundMessageDeserializer Module.
"""


class InboundMessageDeserializer(metaclass=ABCMeta):
    """Deserialize messages received from the platform."""

    def __init__(self, inbound_topics):
        """
        Set inbound topics

        :param inbound_topics: List of topics where messages will be delivered to
        :type inbound_topics: list
        """
        self.inbound_topics = inbound_topics

    @abstractmethod
    def is_actuation_command(self, message):
        """
        Check if message is actuation command

        :param message: The message received
        :type message: wolk.models.InboundMessage.InboundMessage
        :returns: actuation_command
        :rtype: bool
        """
        pass

    @abstractmethod
    def is_firmware_command(self, message):
        """
        Check if message is firmware command

        :param message: The message received
        :type message: wolk.models.InboundMessage.InboundMessage
        :returns: firmware_command
        :rtype: bool
        """
        pass

    @abstractmethod
    def is_file_chunk(self, message):
        """
        Check if message is file chunk

        :param message: The message received
        :type message: wolk.models.InboundMessage.InboundMessage
        :returns: file_chunk
        :rtype: bool
        """
        pass

    @abstractmethod
    def is_configuration(self, message):
        """
        Check if message is configuration

        :param message: The message received
        :type message: wolk.models.InboundMessage.InboundMessage
        :returns: configuration
        :rtype: bool
        """
        pass

    @abstractmethod
    def deserialize_actuator_command(self, message):
        """
        Deserialize the message into an ActuatorCommand.

        :param message: The message received
        :type message: wolk.models.InboundMessage.InboundMessage
        :returns: actuation
        :rtype: wolk.models.ActuatorCommand.ActuatorCommand
        """
        pass

    @abstractmethod
    def deserialize_firmware_command(self, message):
        """
        Deserialize the message into a FirmwareCommand.

        :param message: The message received
        :type message: wolk.models.InboundMessage.InboundMessage
        :returns: firmware_command
        :rtype: wolk.models.FirmwareCommand.FirmwareCommand
        """
        pass

    @abstractmethod
    def deserialize_firmware_chunk(self, message):
        """
        Deserialize the message into a tuple of bytes.

        :param message: The message received
        :type message: wolk.models.InboundMessage.InboundMessage
        :returns: (previous_hash, chunk, chunk_hash)
        :rtype: (bytes, bytes, bytes)
        """
        pass

    @abstractmethod
    def deserialize_configuration_command(self, message):
        """
        Deserialize the message into a ConfigurationCommand.

        :param message: The message received
        :type message: wolk.models.InboundMessage.InboundMessage
        :returns: configuration
        :rtype: wolk.models.ConfigurationCommand.ConfigurationCommand
        """
        pass
